import re

# HTML
html_comments_oneline = re.compile(r'<!--.*?-->')
html_inline_css = re.compile(r'<style.*?>.*?</style>',
                             re.I|re.MULTILINE|re.DOTALL)
quoted_numerical_attribute = re.compile(r"(\S+=(\"|')\d+(\"|'))", re.MULTILINE|re.DOTALL)
any_tag = re.compile(r"<.*?>", re.I|re.MULTILINE|re.DOTALL)
excess_whitespace = re.compile(r' \s+|\s +', re.MULTILINE)

# CSS
css_comments = re.compile(r'/\*.*?\*/', re.MULTILINE|re.DOTALL)

# Define the syntax options we accept
HTML = 'html'
XHTML = 'xhtml'
CSS = 'css'

OK_SYNTAX = (HTML, XHTML, CSS)

def acceptableSyntax(syntax):
    """ return the syntax as we recognize it or None """
    syntax = str(syntax).lower().strip().replace(' ','').replace('-','')
    syntax = syntax.replace('stylesheet','css') # allow for alias
    if syntax in OK_SYNTAX:
	return syntax
    else:
	return None

def slimmer(code, syntax=XHTML):
    """ wrap all function we have """
    if syntax == XHTML:
	return _xhtml_slimmer(code)
    elif syntax == HTML:
	return _html_slimmer(code)
    elif syntax == CSS:
	return _css_slimmer(code)
    
def _css_slimmer(css):
    """ remove repeating whitespace ( \t\n) """

    css = css_comments.sub('', css)
    css = re.sub(r'\s\s+', '', css)
    css = re.sub(r'\s+{','{', css)
    css = re.sub(r'\s}','}', css)
    css = re.sub(r'}','}\n', css)
    css = re.sub(r'>\s+', '>\n', css)
    css = re.sub(r'\s+<', '\n<', css)
    
    return css.strip()

def _xhtml_slimmer(xhtml):
    # optimize inline CSS
    return _html_slimmer(xhtml, unquote_attributes=0)



def _html_slimmer(html, unquote_attributes=1):
    """ Optimize like XHTML but go one step further """
    # 1. optimize inline CSS
    for styletag in html_inline_css.findall(html):
	html = html.replace(styletag, css_slimmer(styletag))
	
    # 2. Remove excessive whitespace between tags
    html = re.sub(r'>\s+<','><', html)
    
    # 3. Remove oneline comments
    html = html_comments_oneline.sub('', html)
    
    # 4. In every tag, remove quotes on numerical attributes and all
    # excessive whitespace
    for tag in any_tag.findall(html):
	original = tag
	distinct_matches=[]
	if unquote_attributes:
	    quoted_attributes = quoted_numerical_attribute.findall(tag)
	else:
	    quoted_attributes = []
	for e in quoted_attributes:
	    if e[0] not in distinct_matches:
		# keep track like this to avoid unnecessary duplicate replaces
		distinct_matches.append(e[0])
		tag = tag.replace(e[0], e[0].replace(e[1],''))
		
	# remove >1 repeating whitespace
	tag = excess_whitespace.sub(' ', tag)
	
	# has the tag been improved?
	if original != tag:
	    html = html.replace(original, tag)
    
    return html.strip()


## ----- Some fancier names
##

def css_slimmer(css):
    return _css_slimmer(css)

def xhtml_slimmer(xhtml):
    return _xhtml_slimmer(xhtml)

def html_slimmer(html):
    return _html_slimmer(html)

    
